define([
    // dojo
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/when",
    "dojo/Deferred",

    // EPi Framework
    "epi/shell/widget/_ModelBindingMixin",

    // EPi CMS
    "epi-cms/widget/LinkEditor",
    "epi-cms/widget/_SelectorBase",
    "epi-cms/core/PermanentLinkHelper",
    "epi/i18n!epi/cms/nls/episerver.cms.widget.editlink"
], function (
    // dojo
    declare,
    lang,
    when,
    Deferred,

    // EPi Framework
    _ModelBindingMixin,

    // EPi CMS
    LinkEditor,
    _SelectorBase,
    PermanentLinkHelper,
    res
) {
    return declare([_SelectorBase, _ModelBindingMixin], {
        // summary:
        //    Represents the widget to edit PropertyURL.
        // tags:
        //    internal

        resource: res,

        baseClass: "epi-urlSelector",

        // LinkHelper to retrieve the link info
        linkHelper: PermanentLinkHelper,

        postMixInProperties: function () {
            this.inherited(arguments);

            if (!this.model && this.modelClassName) {
                var modelClass = declare(this.modelClassName);
                this.model = new modelClass();
            }

            this.actionsNodeOptions = {
                settings: {
                    label: this.resource.buttonlabel
                }
            };
        },

        startup: function () {
            // summary:
            //      Overridden to reset input field.

            if (this._started) {
                return;
            }

            this.inherited(arguments);
            !this.value && this.set("value", null);
        },

        isValid: function () {
            // summary:
            //    Check if widget's value is valid.
            // tags:
            //    protected

            return (!this.required || (!this.get("isEmpty"))); // Not required or have some value.
        },

        _setupDialogSettings: function () {
            // summary:
            //      Setup settings for dialog.
            // tags:
            //      protected virtual

            this.inherited(arguments);

            this.dialogSettings = Object.assign(this.dialogSettings, {
                title: this._getTitle(),
                dialogContentClass: LinkEditor,
                dialogClass: "epi-dialog-portrait",
                destroyOnHide: false,
                defaultActionsVisible: false
            });
        },

        _setupContentSelectorDialogSettings: function () {
            this.inherited(arguments);

            this.contentSelectorDialogSettings = Object.assign(this.contentSelectorDialogSettings, {
                modelType: this.metadata.additionalValues["modelType"],
                hiddenFields: ["text", "title", "target", "language"],
                getCurrentValue: function () {
                    return { href: this.get("value") };
                }.bind(this)
            });
        },

        _onDialogExecute: function (linkObj) {
            //summary:
            //    Handle dialog close through executing OK, Cancel, Delete commands
            // tags:
            //    protected

            // we're only interested in setting the link
            this.set("value", linkObj.href);
        },

        _setValueAttr: function (/* String*/ value) {
            //summary:
            //    Value's setter.
            //
            // value: [String]
            //    A string as link value
            //    Value to be set.
            //
            // tags:
            //  protected

            if (!value) {
                this.set("isEmpty", true);

                this._setValueAndFireOnChange(null);
                return;
            }

            this.set("isEmpty", false);
            this._setValueAndFireOnChange(value);
        },

        _setValueAndFireOnChange: function (/* String */ value) {
            //summary:
            //    Sets the value internally and fires onChange if the value differs than the current value
            //
            // value: [String]
            //    A string as link value
            //    Value to be set.
            //
            // tags:
            //  private

            var currentLink = this.get("value");
            this._set("value", value);

            // detect whether to invoke onChange or not
            var triggerOnChange = true;

            if (!currentLink && !value) {
                triggerOnChange = false;
            } else if (value && value === currentLink) {
                triggerOnChange = false;
            }

            if (triggerOnChange) {
                this.onChange(value);
            }

            // update the displayNode with link name
            // There might be an ongoing getLinkName request, cancel it before starting a new one
            if (this._valueChangedPromise) {
                this._valueChangedPromise.cancel("");
            }

            if (value) {
                this._valueChangedPromise = this._getLinkName(value);
                this._valueChangedPromise.then(lang.hitch(this, this._updateDisplayNode));
            } else {
                this._updateDisplayNode(null);
            }

        },

        _getLinkName: function (/* String */ value) {
            // summary:
            //      Retrieves the name of given link value. If name not found then returns back the same provided value
            //
            // value: [String]
            //    A string as link value
            //
            // tags:
            //      private

            var def = new Deferred();
            when(this.linkHelper.getContent(value, { allLanguages: true }), lang.hitch(this, function (content) {
                var result = { name: content ? content.name : value };
                if (!def.isCanceled()) {
                    def.resolve(result);
                }
            }));

            return def.promise;
        },

        _getTitle: function () {
            // summary:
            //      Customize base get method for title prop.
            // tags:
            //      protected

            return lang.replace(this.value ? this.resource.title.template.edit : this.resource.title.template.create, this.resource.title.action);
        }
    });
});
