define([
    // dojo
    "dojo/_base/declare",
    "dojo/on",
    "dojo/aspect",

    // diji
    "dijit/Destroyable",

    // epi.shell
    "epi/shell/command/_Command",
    "epi/shell/widget/dialog/ContextAwareDialog"
], function (
    // dojo
    declare,
    on,
    aspect,

    // dijit
    Destroyable,

    // epi.shell
    _Command,
    ContextAwareDialog
) {
    return declare([_Command, Destroyable], {
        // summary:
        //    Default command class used in ContentSelectorPlugAndPlay widget.
        // tags:
        //      private xproduct

        contentSelectorDialogSettings: null,
        dialogSettings: null,

        dialog: null,
        contentSelectorDialog: null,

        onContentSelectorChange: function (/*object*/value) {
            // summary:
            //    Handle content selector inside dialog is changed value.
            // tags:
            //    public callback
        },

        onSearchResultSelected: function (/*object*/value) {
            // summary:
            //    Handle a search result is selected.
            // tags:
            //    public callback
        },

        onDialogExecute: function (/*object*/value) {
            // summary:
            //    Handle dialog execute
            // tags:
            //    public callback
        },

        onDialogShow: function () {
            // summary:
            //    Handle dialog show
            // tags:
            //    public callback
        },

        onDialogHide: function () {
            // summary:
            //    Handle dialog hide
            // tags:
            //    public callback
        },

        _execute: function () {
            this._buildDialog();
            var currentValue = this.contentSelectorDialogSettings.getCurrentValue();
            currentValue && this.contentSelectorDialog.set("value", currentValue);
            !currentValue && !this.contentSelectorDialog.setInitialValue && this.contentSelectorDialog.set("value", undefined);
            this.dialog.definitionConsumer.setItemProperty(this.dialog._okButtonName, "disabled", !currentValue);
            this.dialog.show();
            !currentValue && this.contentSelectorDialog.setInitialValue && this.contentSelectorDialog.setInitialValue();
        },

        _buildDialog: function () {
            // summary:
            //    Create content tree dialog
            // tags:
            //    protected

            // Verifies that the dialog instance and its dom node existing or not
            if (this.dialog && this.dialog.domNode) {
                return;
            }

            this.contentSelectorDialog = new this.dialogSettings.dialogContentClass(this.contentSelectorDialogSettings);

            this.dialog = new ContextAwareDialog(Object.assign({ content: this.contentSelectorDialog }, this.dialogSettings));

            this.dialog.own(this.contentSelectorDialog);
            var callbackWithValue = function (callbackName, closeDialog) {
                var value = this.contentSelectorDialog.get("value");
                callbackName && this[callbackName].call(this, value);
                this.dialog && this.dialog.definitionConsumer && this.dialog.definitionConsumer.setItemProperty(this.dialog._okButtonName, "disabled", !value);
                closeDialog && this.dialog && this.dialog.hide();
            }.bind(this);
            this.own(
                this.dialog,
                on(this.contentSelectorDialog, "data-changed", function (value) {
                    this.dialog.definitionConsumer.setItemProperty(this.dialog._okButtonName, "disabled", !value);
                }.bind(this)),
                aspect.after(this.contentSelectorDialog, "onChange", callbackWithValue.bind(this, "onContentSelectorChange", false), true),
                aspect.after(this.contentSelectorDialog, "onSearchResultSelected", callbackWithValue.bind(this, "onSearchResultSelected", true), true),
                aspect.after(this.dialog, "onExecute", callbackWithValue.bind(this, "onDialogExecute"), true),
                aspect.after(this.dialog, "onHide", this.onDialogHide.bind(this), true),
                aspect.after(this.dialog, "onShow", this.onDialogShow.bind(this), true)
            );

            this.dialog.startup();
        }
    });
});
