define([], function () {
    // tags:
    //      internal

    var console = window.console;
    if (!console) {
        return;
    }

    var methods = [
        "log",
        "debug",
        "info",
        "warn",
        "error"
    ];

    var excludes = [];

    function check(message) {
        return excludes.some(function (excludePattern) {
            return message.indexOf(excludePattern) !== -1;
        });
    }

    function intercept(method) {
        var original = console[method];
        console[method] = function () {
            if (original.apply) {
                // Do this for normal browsers
                if (arguments[0] && check(arguments[0].toString())) {
                    return;
                }
                original.apply(console, arguments);
            } else {
                // skip in IE
                var message = Array.prototype.slice.apply(arguments).join(" ");
                original(message);
            }
        };
    }

    methods.forEach(function (method) {
        intercept(method);
    });

    return function (message) {
        if (excludes.indexOf(message) !== -1) {
            return;
        }

        excludes.push(message);
    };
});
